from __future__ import annotations

import json
import logging
from pathlib import Path
from typing import Any

from django.core.management.base import BaseCommand, CommandError, CommandParser
from django.db import transaction
from django.utils import timezone
from django.utils.dateparse import parse_datetime

from core.models import Benefit, DropCampaign, Game, Organization, TimeBasedDrop

# Configure logger for this module
logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """Imports Twitch Drop campaign data from a directory of JSON files into the database."""

    help = "Imports Twitch Drop campaign data from a directory of JSON files."

    def add_arguments(self, parser: CommandParser) -> None:  # noqa: PLR6301
        """Add command line arguments for the command.

        Args:
            parser (CommandParser): The command parser to add arguments to.
        """
        parser.add_argument("json_dir", type=str, help="Path to the directory containing Twitch Drop JSON files")
        parser.add_argument(
            "--pattern",
            type=str,
            default="*.json",
            help="Glob pattern to filter JSON files (default: *.json)",
        )

    def handle(self, *args: tuple[str, ...], **options: dict[str, Any]) -> None:
        """Import data to DB from multiple JSON files in a directory.

        Args:
            *args (tuple[str, ...]): Positional arguments passed to the command.
            **options (dict[str, Any]): Keyword arguments passed to the command.

        Raises:
            CommandError: If the directory does not exist or is not a directory.
        """
        self.stdout.write(self.style.NOTICE("Starting import process..."))
        self.stdout.write(self.style.NOTICE(f"Arguments: {args}; Options: {options}"))

        json_dir_path = options["json_dir"]
        pattern = options["pattern"]
        dir_path = Path(json_dir_path)  # pyright: ignore[reportArgumentType]

        # Check if directory exists
        if not dir_path.exists():
            msg = f"Error: Directory not found at {json_dir_path}"
            logger.error(msg)
            raise CommandError(msg)

        if not dir_path.is_dir():
            msg = f"Error: {json_dir_path} is not a directory"
            logger.error(msg)
            raise CommandError(msg)

        # Find JSON files in the directory
        json_files = list(dir_path.glob(pattern))  # pyright: ignore[reportArgumentType]

        if not json_files:
            msg = f"No JSON files found in {json_dir_path} matching pattern '{pattern}'"
            self.stdout.write(self.style.WARNING(msg))
            logger.warning(msg)
            return

        self.stdout.write(f"Found {len(json_files)} JSON files to process in {json_dir_path}")
        logger.info("Found %s JSON files to process in %s", len(json_files), json_dir_path)

        # Tracking statistics
        total_stats = {
            "files_processed": 0,
            "files_with_errors": 0,
            "games_created": 0,
            "games_updated": 0,
            "orgs_created": 0,
            "orgs_updated": 0,
            "campaigns_created": 0,
            "campaigns_updated": 0,
            "drops_created": 0,
            "drops_updated": 0,
            "benefits_created": 0,
            "benefits_updated": 0,
        }

        # Process each file
        for json_file in sorted(json_files):
            try:
                file_stats = self._process_json_file(json_file)
                total_stats["files_processed"] += 1

                # Update totals
                for key, value in file_stats.items():
                    if key in total_stats:
                        total_stats[key] += value
            except CommandError as e:
                self.stdout.write(self.style.ERROR(str(e)))
                total_stats["files_with_errors"] += 1
                logger.exception("Error processing %s", json_file)
            except Exception as e:
                msg = f"Unexpected error processing {json_file}: {e}"
                self.stdout.write(self.style.ERROR(msg))
                logger.exception(msg)
                total_stats["files_with_errors"] += 1

        # Print summary
        self.stdout.write(self.style.SUCCESS("Import process completed."))
        self.stdout.write(f"Files processed: {total_stats['files_processed']}")
        self.stdout.write(f"Files with errors: {total_stats['files_with_errors']}")
        self.stdout.write(f"Games created: {total_stats['games_created']}, updated: {total_stats['games_updated']}")
        self.stdout.write(
            f"Organizations created: {total_stats['orgs_created']}, updated: {total_stats['orgs_updated']}",
        )
        self.stdout.write(
            f"Campaigns created: {total_stats['campaigns_created']}, updated: {total_stats['campaigns_updated']}",
        )
        self.stdout.write(f"Drops created: {total_stats['drops_created']}, updated: {total_stats['drops_updated']}")
        self.stdout.write(
            f"Benefits created: {total_stats['benefits_created']}, updated: {total_stats['benefits_updated']}",
        )

        logger.info(
            "Import completed. Processed: %s, Errors: %s",
            total_stats["files_processed"],
            total_stats["files_with_errors"],
        )

    def _process_json_file(self, json_file: Path) -> dict[str, int]:  # noqa: C901, PLR0912, PLR0914, PLR0915
        """Process a single JSON file and import its data to the database.

        Args:
            json_file: Path object pointing to the JSON file.

        Returns:
            Dict with counts of created and updated records.

        Raises:
            CommandError: If the JSON file cannot be decoded or if there are
                          issues with the JSON structure.
        """
        self.stdout.write(f"Processing file: {json_file}")
        logger.info("Processing file: %s", json_file)

        # Initialize stats for this file
        stats = {
            "games_created": 0,
            "games_updated": 0,
            "orgs_created": 0,
            "orgs_updated": 0,
            "campaigns_created": 0,
            "campaigns_updated": 0,
            "drops_created": 0,
            "drops_updated": 0,
            "benefits_created": 0,
            "benefits_updated": 0,
        }

        try:
            with json_file.open(encoding="utf-8") as f:
                raw_data = json.load(f)
        except FileNotFoundError as e:
            msg = f"Error: File not found at {json_file}"
            logger.exception(msg)
            raise CommandError(msg) from e
        except json.JSONDecodeError as e:
            msg = f"Error: Could not decode JSON from {json_file}"
            logger.exception(msg)
            raise CommandError(msg) from e
        except Exception as e:
            msg = f"Error reading file: {e}"
            logger.exception(msg)
            raise CommandError(msg) from e

        # Navigate to the relevant part of the JSON structure
        try:
            user_data = raw_data.get("data", {}).get("user", {})
            campaign_data = user_data.get("dropCampaign")

            if not campaign_data:
                msg = f"Error: 'dropCampaign' key not found or is null in the JSON data: {json_file}"
                logger.error(msg)
                raise CommandError(msg)

        except AttributeError as e:
            msg = f"Error: Unexpected JSON structure in {json_file}. Could not find 'data' or 'user'."
            logger.exception(msg)
            raise CommandError(msg) from e

        # Process the campaign data within a transaction
        try:  # noqa: PLR1702
            # Use a transaction to ensure atomicity
            with transaction.atomic():
                # --- 1. Process Game ---
                game_data = campaign_data.get("game")
                game_obj = None
                if game_data and game_data.get("id"):
                    game_defaults = {
                        "slug": game_data.get("slug"),
                        "display_name": game_data.get("displayName"),
                    }
                    game_obj, created = Game.objects.update_or_create(
                        game_id=game_data["id"],
                        defaults=game_defaults,
                    )
                    if created:
                        self.stdout.write(self.style.SUCCESS(f"Created Game: {game_obj.display_name}"))
                        stats["games_created"] += 1
                    else:
                        self.stdout.write(f"Updated/Found Game: {game_obj.display_name}")
                        stats["games_updated"] += 1
                else:
                    self.stdout.write(self.style.WARNING("No game data found in campaign."))
                    logger.warning("No game data found in campaign.")

                # --- 2. Process Owner Organization ---
                owner_data = campaign_data.get("owner")
                owner_obj = None
                if owner_data and owner_data.get("id"):
                    owner_defaults = {"name": owner_data.get("name")}
                    owner_obj, created = Organization.objects.update_or_create(
                        org_id=owner_data["id"],
                        defaults=owner_defaults,
                    )
                    if created:
                        self.stdout.write(self.style.SUCCESS(f"Created Organization: {owner_obj.name}"))
                        stats["orgs_created"] += 1
                    else:
                        self.stdout.write(f"Updated/Found Organization: {owner_obj.name}")
                        stats["orgs_updated"] += 1
                else:
                    self.stdout.write(self.style.WARNING("No owner organization data found in campaign."))
                    logger.warning("No owner organization data found in campaign.")

                # --- 3. Process Drop Campaign ---
                campaign_id = campaign_data.get("id")
                if not campaign_id:
                    msg = "Error: Campaign ID is missing."
                    logger.error(msg)
                    raise CommandError(msg)  # noqa: TRY301

                start_at_str = campaign_data.get("startAt")
                end_at_str = campaign_data.get("endAt")
                start_at_dt = parse_datetime(start_at_str) if start_at_str else None
                end_at_dt = parse_datetime(end_at_str) if end_at_str else None

                if not start_at_dt or not end_at_dt:
                    self.stdout.write(self.style.WARNING(f"Campaign {campaign_id} missing start or end date."))
                    logger.warning("Campaign %s missing start or end date.", campaign_id)
                    # Decide if you want to skip or handle this case differently

                campaign_defaults = {
                    "name": campaign_data.get("name"),
                    "description": campaign_data.get("description"),
                    "status": campaign_data.get("status"),
                    "start_at": start_at_dt,
                    "end_at": end_at_dt,
                    "image_url": campaign_data.get("imageURL"),
                    "details_url": campaign_data.get("detailsURL"),
                    "account_link_url": campaign_data.get("accountLinkURL"),
                    "game": game_obj,
                    "owner": owner_obj,
                }
                campaign_obj, created = DropCampaign.objects.update_or_create(
                    campaign_id=campaign_id,
                    defaults=campaign_defaults,
                )
                if created:
                    self.stdout.write(self.style.SUCCESS(f"Created Campaign: {campaign_obj.name}"))
                    stats["campaigns_created"] += 1
                else:
                    self.stdout.write(f"Updated/Found Campaign: {campaign_obj.name}")
                    stats["campaigns_updated"] += 1

                # --- 4. Process Time Based Drops ---
                time_drops_data = campaign_data.get("timeBasedDrops", [])
                if not time_drops_data:
                    self.stdout.write(self.style.NOTICE("No timeBasedDrops found in campaign data."))
                    logger.info("No timeBasedDrops found in campaign data.")

                for drop_data in time_drops_data:
                    drop_id = drop_data.get("id")
                    if not drop_id:
                        self.stdout.write(self.style.WARNING("Skipping drop with missing ID."))
                        logger.warning("Skipping drop with missing ID.")
                        continue

                    drop_start_str = drop_data.get("startAt")
                    drop_end_str = drop_data.get("endAt")
                    drop_start_dt = parse_datetime(drop_start_str) if drop_start_str else None
                    drop_end_dt = parse_datetime(drop_end_str) if drop_end_str else None

                    if not drop_start_dt or not drop_end_dt:
                        self.stdout.write(self.style.WARNING(f"Drop {drop_id} missing start or end date. Skipping."))
                        logger.warning("Drop %s missing start or end date. Skipping.", drop_id)
                        continue

                    drop_defaults = {
                        "name": drop_data.get("name"),
                        "start_at": drop_start_dt,
                        "end_at": drop_end_dt,
                        "required_minutes_watched": drop_data.get("requiredMinutesWatched"),
                        "campaign": campaign_obj,
                        # Add required_subs if needed: 'required_subs': drop_data.get('requiredSubs', 0)
                    }

                    drop_obj, created = TimeBasedDrop.objects.update_or_create(drop_id=drop_id, defaults=drop_defaults)
                    if created:
                        self.stdout.write(self.style.SUCCESS(f"  Created Time Drop: {drop_obj.name}"))
                        stats["drops_created"] += 1
                    else:
                        self.stdout.write(f"  Updated/Found Time Drop: {drop_obj.name}")
                        stats["drops_updated"] += 1

                    # --- 5. Process Benefits within the Drop ---
                    benefits_data = drop_data.get("benefitEdges", [])
                    benefit_objs = []
                    for edge in benefits_data:
                        benefit_info = edge.get("benefit")
                        if not benefit_info or not benefit_info.get("id"):
                            self.stdout.write(self.style.WARNING("  Skipping benefit edge with missing data or ID."))
                            logger.warning("Skipping benefit edge with missing data or ID.")
                            continue

                        benefit_id = benefit_info["id"]
                        benefit_game_data = benefit_info.get("game")
                        benefit_owner_data = benefit_info.get("ownerOrganization")
                        benefit_created_str = benefit_info.get("createdAt")
                        benefit_created_dt = parse_datetime(benefit_created_str) if benefit_created_str else None

                        benefit_game_obj = None
                        if benefit_game_data and benefit_game_data.get("id"):
                            # Assuming game ID is unique, get it directly
                            try:
                                benefit_game_obj = Game.objects.get(game_id=benefit_game_data["id"])
                            except Game.DoesNotExist:
                                warning_msg = (
                                    f"Game {benefit_game_data.get('name')} for benefit {benefit_id} "
                                    f"not found. Creating minimally."
                                )
                                self.stdout.write(self.style.WARNING(f"    {warning_msg}"))
                                logger.warning(warning_msg)
                                # Optionally create a minimal game entry here if desired
                                benefit_game_obj, game_created = Game.objects.update_or_create(
                                    game_id=benefit_game_data["id"],
                                    defaults={"display_name": benefit_game_data.get("name", "Unknown Game")},
                                )
                                if game_created:
                                    stats["games_created"] += 1

                        benefit_owner_obj = None
                        if benefit_owner_data and benefit_owner_data.get("id"):
                            try:
                                benefit_owner_obj = Organization.objects.get(org_id=benefit_owner_data["id"])
                            except Organization.DoesNotExist:
                                warning_msg = (
                                    f"Organization {benefit_owner_data.get('name')} for benefit {benefit_id} "
                                    f"not found. Creating minimally."
                                )
                                self.stdout.write(self.style.WARNING(f"    {warning_msg}"))
                                logger.warning(warning_msg)
                                benefit_owner_obj, org_created = Organization.objects.update_or_create(
                                    org_id=benefit_owner_data["id"],
                                    defaults={"name": benefit_owner_data.get("name", "Unknown Org")},
                                )
                                if org_created:
                                    stats["orgs_created"] += 1

                        benefit_defaults = {
                            "name": benefit_info.get("name"),
                            "image_asset_url": benefit_info.get("imageAssetURL"),
                            "entitlement_limit": edge.get("entitlementLimit", 1),  # Get limit from edge
                            "distribution_type": benefit_info.get("distributionType"),
                            "twitch_created_at": benefit_created_dt or timezone.now(),  # Provide a default if missing
                            "game": benefit_game_obj,
                            "owner_organization": benefit_owner_obj,
                            "is_ios_available": benefit_info.get("isIosAvailable", False),
                        }

                        benefit_obj, b_created = Benefit.objects.update_or_create(
                            benefit_id=benefit_id,
                            defaults=benefit_defaults,
                        )
                        if b_created:
                            self.stdout.write(self.style.SUCCESS(f"    Created Benefit: {benefit_obj.name}"))
                            stats["benefits_created"] += 1
                        else:
                            self.stdout.write(f"    Updated/Found Benefit: {benefit_obj.name}")
                            stats["benefits_updated"] += 1
                        benefit_objs.append(benefit_obj)

                    # Set the ManyToMany relationship for the drop
                    if benefit_objs:
                        drop_obj.benefits.set(benefit_objs)
                        drops_msg = f"Associated {len(benefit_objs)} benefits with drop {drop_obj.name}."
                        self.stdout.write(f"  {drops_msg}")
                        logger.info("Associated %s benefits with drop %s.", len(benefit_objs), drop_obj.name)

        except KeyError as e:
            msg = f"Error: Missing expected key in JSON data - {e}"
            logger.exception(msg)
            raise CommandError(msg) from e
        except Exception as e:
            # The transaction will be rolled back automatically on exception
            msg = f"An error occurred during import: {e}"
            logger.exception(msg)
            raise CommandError(msg) from e

        # Return statistics from this file
        return stats
